(***************************************************************************

  toupper - Change file names to lower or upper case

  Copyright 2001-2002 by MSDatec

    This code is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This code is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************)



program toupper;

uses windows,TULib;

{$R *.RES}





procedure ShowVersion;
begin

  writeln(CProgramName+' '+CVersionStr);
  writeln('Copyright 2002 by MSDatec'+#10#13
         +#9+CProgramName+' comes with NO WARRANTY, to the extent permitted'+#10#13
         +#9+'by law. You may redistribute copies of '+CProgramName+' under the'+#10#13
         +#9+'terms of the GNU General Public License.'+#10#13
         +#9+'For more information about these matters, see the'+#10#13
         +#9+'files named COPYING.'+#10#13
         +'Report bugs to ms@msdatec.de');
end;


procedure Help;
  procedure WriteOptLine(OptStr:String;msg:string);
  const CIndent = 11;
  var i:byte;
  begin
    write('-',OptStr);
    for i:=1 to CIndent-length(OptStr) do write(' ');
    writeln(msg);
  end;
begin
  ShowVersion;
  writeln('Call with: '+CProgramName+' [options] <pattern>');
  writeln('Options:');
  WriteOptLine(COptChar_DOSOnly,     'process DOS-formed filenames (8.3) only');
//  WriteOptLine(COptChar_force,       'force deleting of existing files (except directories)');
  WriteOptLine('h',                  'show this help');
  WriteOptLine(COptChar_IgnoreErrors,'don''t abort on errors');
//  WriteOptLine(COptChar_Interactive, 'interactive mode');
  WriteOptLine(COptChar_Lower,       'force lowering of filenames');
  WriteOptLine(COptChar_Recursive,   'recursive');
  WriteOptLine(COptChar_Upper,       'force uppering of filenames (default)');
  WriteOptLine(COptChar_Verbose,     'enable verbose mode (use multiple to increase level)');
  WriteOptLine(COptChar_Version,     'show version');
  {$IFDEF DEBUG}
  readln;
  {$ENDIF }
end;


{$IFDEF DEBUG}
  procedure WriteFlag(Name:string;state:boolean);
  begin
    write(name,': ');
    if state then writeln('on') else writeln('off')
  end;
{$ENDIF}



function IsOptChar(c:char):boolean;
begin
  result:=(c='-') or (c='/')
end;


function IsOption(s:string;pOptChar:PChar):boolean;
begin
  if (length(s)>=2) AND (IsOptChar(s[1])) and not IsOptChar(s[2]) then
    begin
      if pOptChar<>NIL then pOptChar^:=s[2];
      result:=true
    end
  else
    result:=false;
end;


procedure renameLoop(Dir,pattern:string;const level:integer);
var NewName:String;
const CLowerUpperStrs : array[boolean] of string[5] = ('upper','lower');

  function TreatThisFile(const filename:String):boolean;
  begin
    result:=false;
    if OptDOSNamesOnly then
      if not IsDosFilename(filename) then
        begin
          VerboseWrite(dir+FileName +': does not follow DOS-name convention',CVerboseUser,true);
          exit;
        end;
    setLength(NewName,length(filename));
    move(filename[1],NewName[1],length(filename));
    if OptLower then
      CharLower(pchar(NewName))
    else
      CharUpper(pchar(NewName));
    if NewName=FileName then // do not treat
      VerboseWrite(dir+FileName+': already '+CLowerUpperStrs[OptLower],CVerboseUser,true)
    else
      result:=true;
  end;


var FindData: TWin32FindData;
    FindHandle: THandle;
begin
  if OptRecursive then
    begin
      FindHandle:=FindFirstFile(PChar(dir+'*.*'),FindData);
      if FindHandle<>INVALID_HANDLE_VALUE then
        repeat
          if (FindData.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY)=FILE_ATTRIBUTE_DIRECTORY then
            if (pchar(@FindData.cFileName)<>'.') and (FindData.cFileName<>'..') then
              begin
                VerboseWrite('treating directory '+Dir+FindData.cFileName+'...',CVerboseDetailed,true);
                renameLoop(Dir+FindData.cFileName+'\',pattern,level+1)
              end
            else
              VerboseWrite('ignoring directory '+pchar(@FindData.cFileName),CVerboseDebug,true)
        until not FindNextFile(FindHandle,FindData);
      Findclose(FindHandle);
    end;

  { special case: if the only parameter of the program is a directory, *pattern*
    is empty and *dir* is the parameter. Because *FindFirstFile* cannot handle
    path names ending with '\', split *dir* again in upperdir(dir) and set
    pattern to the name of the directory: }
  if Level=0 then
    if pattern='' then
      if dir<>'' then
        if dir[length(dir)]='\' then
          begin
            setlength(dir,length(dir)-1);
            pattern:=ExtractFilename(dir,false);
            dir:=dirBS(ExtractFilePath(dir,false));
          end;


  FindHandle:=FindFirstFile(PChar(dir + pattern),FindData);
  if FindHandle<>INVALID_HANDLE_VALUE then
    begin
      repeat
        if (pchar(@FindData.cFileName)<>'.') and (FindData.cFileName<>'..') and TreatThisFile(FindData.cFileName) then
          begin
            VerboseWrite(PChar(dir+FindData.cFileName)+': moving to '+NewName+'...',CVerboseUser,false);
            if not MoveFile(PChar(dir+FindData.cFileName),pchar(dir+NewName)) then
              begin
                if not OptIgnoreErrors then Findclose(FindHandle);
                ErrorWrite(' '+pchar(dir+NewName),getLastError,not OptIgnoreErrors)
              end
            else
              VerboseWrite('done',CVerboseUser,true)
          end
        else
          VerboseWrite('ignoring file '+FindData.cFileName,CVerboseDebug,true)

      until not FindNextFile(FindHandle,FindData);
      Findclose(FindHandle);
    end
  else
    VerboseWrite(dir+pattern+': no files found',CVerboseUser,true);

end;


var OptIndex:integer;
    optChar:char;
    pattern:string='';
    OutPutDir:string='';
begin
  if paramcount=0 then begin help; halt(1);end;
  OptIndex:=1;
  while OptIndex<=paramcount do
    begin
      if IsOption(paramstr(OptIndex),@OptChar) then
        case OptChar of

          {COptChar_Interactive:
            OptInteractive:=true;}

          'h','?':
            begin Help;halt(0);end;

          COptChar_DOSOnly:
            OptDOSNamesOnly:=true;

          COptChar_Verbose:
            inc(OptVerbose);

          COptChar_Upper:
            OptLower:=false;

          COptChar_Lower:
            OptLower:=true;

          COptChar_Recursive:
            OptRecursive:=true;

          COptChar_IgnoreErrors:
            OptIgnoreErrors:=true;

          COptChar_Version:
            begin showversion;halt(0);end;
        else
          error(CErr_cmdline,'unknown option: -'+OptChar+CStrHelpHint)
        end // case
      else
        if pattern<>'' then
          error(CErr_cmdline,'unknown argument: '+paramstr(OptIndex)+CStrHelpHint)
        else
          pattern:=ExpandFileName(paramstr(OptIndex));
      inc(OptIndex);
    end;
    if pattern='' then error(CErr_cmdline,'no pattern given');

{$IFDEF DEBUG}
    writeln('Options:');
    //WriteFlag('CFlag_Interactive',OptInteractive);
    WriteFlag('CFlag_Recursive',OptRecursive);
    WriteFlag('CFlag_ToLower',OptLower);
    //WriteFlag('CFlag_force',Optforce);
    WriteFlag('CFlag_DOSNamesOnly',OptDOSNamesOnly);
    Writeln('Verboselevel: ',OptVerbose);

    writeln('pattern: "',pattern,'"');
    writeln('dir in pattern: "',ExtractFilePath(pattern),'"');
    writeln('file name in pattern: "',ExtractFileName(pattern),'"');


    write('Press RETURN...');
    readln;
  {$ENDIF}
  renameLoop(dirBS(ExtractFilePath(pattern,true)),ExtractFileName(pattern,true),0);
  {$IFDEF DEBUG}
    write('Program successfully terminated. Press RETURN...');
    readln;
  {$ENDIF}
end.
