unit TULib;

interface

const CStrHelpHint      = ' (try -h for help)';

      CVerboseUser     = 1; // show what's going on
      CVerboseDetailed = 2; // show even more
      CVerboseDebug    = 3; // show all

const CVersionStr='0.1';
      CProgramName='ToUpper';
      CErrorTextFileName = 'TOUPR-EN.NLS';


      COptChar_DOSOnly      = 'D';
      //COptChar_Help       = 'h'; hardcoded
      COptChar_IgnoreErrors = 'i';
      COptChar_Lower        = 'l';
      COptChar_Recursive    = 'r';
      COptChar_Upper        = 'U';
      COptChar_Verbose      = 'v';
      COptChar_Version      = 'V';
      { currently not supported:
      COptChar_Interactive  = 'i';
      COptChar_force        = 'f';}

      //CFlag_Interactive     = $01;
      //CFlag_Verbose         = $02;
      //CFlag_Recursive       = $04;
      //CFlag_ToLower         = $08;
      //CFlag_force           = $10;
      //CFlag_DOSNamesOnly    = $20;


var OptDOSNamesOnly: boolean = false;
    {OptInteractive:  boolean = false;
    Optforce:        boolean = false;}
    OptLower:        boolean = false;
    OptRecursive:    boolean = false;
    OptVerbose:      integer = 0;
//    OptIgnore:      integer = 0;
    OptIgnoreErrors: boolean = false;



const CErr_cmdline      = 255;
      CErr_file_missing = 254;
      CErr_userabort    = 253;


TYPE TuInt32 = Cardinal;
     TuInt32DecStr   = string[length('4294967294')];

const cLF          = #10;
      cCR          = #13;
      cCRLF = cCR+cLF;


function  DirBS(Dir:string):string;
procedure Error(const Exitcode:byte;const msg:string);
procedure ErrorWrite(const msg:string;const ErrorCode:integer;const DoHalt:boolean);
function  ExpandFileName(const FileName:string):string;
function  ExtractFilePath(const FileName: string;const CheckPhysical:boolean):string;
function  ExtractFileName(const filename:string;const CheckPhysical:boolean):string;
function  IsDirectoryName(const filename:string):boolean;
function  IsDOSFilename(const filename:string):boolean;
function  NoDirBS(const path:string):string;
function  ProgramDir:String;
function  uInt32ToDecStr(uInt32:TuInt32):TuInt32DecStr;
procedure VerboseWrite(const msg:string;const ShowLevel:integer;const lb:boolean);


implementation

uses windows;

function CharIsWhite(c:char):boolean;assembler;
asm
  CMP  AL,32 // Space
  JE   @IsTrue
  CMP  AL,13 // CR
  JE   @IsTrue
  CMP  AL,10 // LF
  JE   @IsTrue
  CMP  AL,11  // Tab
  JE   @IsTrue
  CMP  AL,12  // CFormFeed
  JE   @IsTrue
  CMP  AL,11 // Vertical Tab
  JE   @IsTrue
  XOR  AL,AL
  STC // store CF if CharIsWhite is used by other asm routine
  JMP  @ende
@IsTrue:
  MOV  AL,true
  CLC // clear CF if CharIsWhite is used by other asm routine
@ende:
END;



function DirBS(Dir:string):string;
begin
  if (Dir<>'') and (dir[length(Dir)]<>'\') then DirBS:=Dir+'\'
  else DirBS:=Dir
end;



procedure Error(const Exitcode:byte;const msg:string);
begin
  writeln('[',CProgramName+'] error: '+msg);
  halt(exitcode);
end;


function ErrorTextGet(const Errorcode:integer;var ErrorText:string):boolean;
const CCommentChar = '#';
var f:text;
    s:string;
    err,AVal,i:integer;
begin
  result:=false;
  VerboseWrite('opening error text file "'+DirBS(ProgramDir)+CErrorTextFileName+'": ', CVerboseDebug,false);
  assign(f,DirBS(ProgramDir)+CErrorTextFileName);
  {$IFOPT I+}
    {$DEFINE SWITCH_ON_R}
    {$R-}
  {$ENDIF}
  reset(f);
  {$IFDEF SWITCH_ON_R}
    {$R+}
  {$ENDIF}

  if IOResult<>0 then
    begin
      VerboseWrite('not found.',CVerboseDebug,true);
      exit; // no file with error messages found...
    end;
  VerboseWrite('done.',CVerboseDebug,true);

  try
    while not eof(f) do
      begin
        readln(f,s);
        if s='' then continue;
        i:=1;
        while (i<=length(s)) and CharIsWhite(s[i]) do
          inc(i);
        if i>length(s) then continue;
        if s[i]=CCommentChar then continue;
        // now look for '='
        i:=pos('=',s);
        if i=0 then continue;
        // is it a numeric value?
        val(copy(s,1,i-1),AVal,err);
        if err<>0 then continue;
        if AVal<>Errorcode then continue;
        result:=true;
        ErrorText:=copy(s,i+1,length(s)-i);
      end;
  finally
    close(f)
  end
end;



procedure ErrorWrite(const msg:string;const ErrorCode:integer;const DoHalt:boolean);
var ErrorText:string;
begin
  if not ErrorTextGet(Errorcode,ErrorText) then
    ErrorText:='[no entry found in '+CErrorTextFileName+']';
  writeln('[',CProgramName+'] error '+uInt32ToDecStr(ErrorCode)+': '+ErrorText+msg);
  if DoHalt then halt(ErrorCode);
end;


function ExpandFileName(const FileName:string):string;
var FName: PChar;
    Buffer: array[0..MAX_PATH - 1] of Char;
begin
  SetString(Result, Buffer, GetFullPathName(PChar(FileName), SizeOf(Buffer), Buffer, FName));
end;



function ExtractHelperIsSurelyDir(const FileName: string;const CheckPhysical:boolean): boolean;
begin
  result:=false;
  if (Filename='.') or (FileName='..') then result:=true
  else
    if CheckPhysical then
      if (pos('*',FileName)=0) and (pos('?',FileName)=0) and (IsDirectoryName(FileName)) then result:=true
end;


function ExtractFilePath(const FileName: string;const CheckPhysical:boolean): string;
var i:integer;
begin
  if ExtractHelperIsSurelyDir(FileName,CheckPhysical) then ExtractFilePath:=filename
  else
    begin
      i:=length(filename);
      while (i>0) and (filename[i]<>'\') and (filename[i]<>':') do dec(i);
      ExtractFilePath:=copy(filename,1,i);
    end
end;



function ExtractFileName(const filename:string;const CheckPhysical:boolean):string;

  function IsDelimiter(c:char):boolean;
  begin
    result:=(c='\') or (c=':')
  end;

var i:integer;
begin
  if ExtractHelperIsSurelyDir(FileName,CheckPhysical) then ExtractFileName:=''
  else
    begin
      i:=length(filename);
      while (i>0) and (not IsDelimiter(filename[i])) do
        dec(i);
      if i=0 then
        ExtractFileName:=filename
      else
        ExtractFileName:=copy(filename,i+1,length(filename)-i);
    end;
end;



function IsDirectoryName(const filename:string):boolean;
var FindData: TWin32FindData;
    FindHandle: THandle;
begin
  if filename='' then result:=false
  else
    if filename[length(filename)]='\' then result:=true
    else
      begin
        FindHandle:=FindFirstFile(pchar(filename),FindData);
        result:=(FindHandle<>INVALID_HANDLE_VALUE)
          and
          ((FindData.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY)=FILE_ATTRIBUTE_DIRECTORY);
        Findclose(FindHandle);
      end;
end;


function IsDOSFilename(const filename:string):boolean;
var i:integer;
const InValidChars='\ :<>,=*?/+|';
begin
  result:=false;
  if length(filename)>8+1+3 then exit;
  for i:=1 to length(InValidChars) do
    if pos(InValidChars[i],FileName)<>0 then exit;

  i:=pos('.',filename);
  if i=0 then
    if length(filename)>8 then exit
    else
  else  // file name with dot
    if i>9 then exit
    else
      if length(Filename)-i>3 then exit;
  result:=true;
end;


function NoDirBS(const path:string):string;
begin
  result:=path;
  if result<>'' then
    if result[length(result)]='\' then
      setlength(result,length(result)-1)
end;


FUNCTION ProgramDir:String;
BEGIN
  {$IFDEF DEBUG}
    ProgramDir:='H:\PROG\PAS\PROGS\UTILS\TOUPPER\CV\BIN'
  {$ELSE}
    ProgramDir:=ExtractFilePath(ParamStr(0),true);
  {$ENDIF}
END;




function uInt32ToDecStr(uInt32:TuInt32):TuInt32DecStr;
asm
  PUSH ECX
  PUSH EDX
  PUSH EDI
  PUSH ESI
  MOV  EDI,@result
  XOR  ECX,ECX    { counter for digits on the stack }
  NOT  ECX        { set to -1 to prevent CX-counter condition in LOOPE }
  MOV  ESI,10     { Division by 10 (in ESI) }

@NON_ZERO:
  XOR   EDX,EDX    { Oberen int von N nach 0 setzen }
  DIV   ESI        { Berechne N/10 und (N mod 10), Ergebnis liegt in EAX, Rest in EDX }
  ADD   DL,'0'     { Rest in Ziffer verwandeln }
  PUSH  DX         { Pushe eine Ziffer auf den Stack }
  OR    EAX,EAX     { Ist N = 0 ? }
  LOOPNE @NON_ZERO
  NOT   ECX
  MOV   AL,CL
  STOSB            { Laengenbyte speichern }
@GetDigitLoop:
  POP   AX         { Hole die Ziffern in umgekehrter Reihenfolge }
  STOSB
  LOOP @GetDigitLoop

  POP  ESI
  POP  EDI
  POP  EDX
  POP  ECX
end;


{$IFDEF DEBUG}
var lines : byte = 1;
{$ENDIF}

procedure VerboseWrite(const msg:string;const ShowLevel:integer;const lb:boolean);
begin
  if ShowLevel>OptVerbose then exit;
  write(msg);
  if lb then
    begin
      {$IFDEF DEBUG}
        if lines>25 then
          begin
            write('Press ENTER...');
            readln;
            lines:=1;
          end
        else
          inc(lines);
      {$ENDIF}
      writeln;
    end;
end;




end.
